@@ -21,3 +21,11 @@ class GeneratePayloadError(CtfboxError):
 
 class HttprawError(CtfboxError):
     pass
+
+
+class ScanError(CtfboxError):
+    pass
+
+
+class RepairError(CtfboxError):
+    pass
